/*
 * Copyright (c) 2005 - 2007, Tranql project contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.tranql.connector.mysql;

import com.mysql.jdbc.jdbc2.optional.MysqlConnectionPoolDataSource;
import org.tranql.connector.jdbc.AbstractLocalDataSourceMCF;
import org.tranql.connector.jdbc.AbstractPooledConnectionDataSourceMCF;

/**
 * ManagedConnectionFactory for connecting to a MySQL database with local (non-XA) transactions.
 *
 * @version $Revision: 663 $ $Date: 2009-09-24 23:48:55 -0700 (Thu, 24 Sep 2009) $
 */
public class LocalMCF extends AbstractPooledConnectionDataSourceMCF<MysqlConnectionPoolDataSource> {
    private String password;

    /**
     * Default constructor for a MySQL Local DataSource.
     *
     */
    public LocalMCF() {
        super(new MysqlConnectionPoolDataSource(), new MysqlExceptionSorter());
    }

    public void setDatabaseName(String dbname) {
        connectionPoolDataSource.setDatabaseName(dbname);
    }

    public String getDatabaseName() {
        return connectionPoolDataSource.getDatabaseName();
    }

    public void setPassword(String pwd) {
        this.password = pwd;
        connectionPoolDataSource.setPassword(pwd);
    }

    public String getPassword() {
        return password;
    }

    public void setPortNumber(Integer pn) {
        connectionPoolDataSource.setPortNumber(pn == null ? 0 : pn.intValue());
    }

    public Integer getPortNumber() {
        return new Integer(connectionPoolDataSource.getPortNumber());
    }

    public void setServerName(String sn) {
        connectionPoolDataSource.setServerName(sn);
    }

    public String getServerName() {
        return connectionPoolDataSource.getServerName();
    }

    public void setUserName(String user) {
        connectionPoolDataSource.setUser(user);
    }

    public String getUserName() {
        return connectionPoolDataSource.getUser();
    }

    /**
     * Equality is defined in terms of the serverName, portNumber and databaseName properties
     *
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof LocalMCF) {
            MysqlConnectionPoolDataSource other = ((LocalMCF) obj).connectionPoolDataSource;
            return equals(connectionPoolDataSource.getServerName(), other.getServerName()) &&
                    connectionPoolDataSource.getPortNumber() == other.getPortNumber() &&
                    equals(connectionPoolDataSource.getDatabaseName(), other.getDatabaseName());
        } else {
            return false;
        }
    }

    private static boolean equals(String a, String b) {
        return a == b || a != null && a.equals(b);
    }

    public int hashCode() {
        return hashCode(connectionPoolDataSource.getServerName()) ^ connectionPoolDataSource.getPortNumber() ^ hashCode(connectionPoolDataSource.getDatabaseName());
    }

    private static int hashCode(String s) {
        return s == null ? 0 : s.hashCode();
    }

    public String toString() {
        return "LocalMCF[" + getDatabaseName() + ']';
    }
}
